/**
 * 
 */
package org.gcube.portlets.user.td.csvexportwidget.client;

import org.gcube.portlets.user.td.gwtservice.client.rpc.TDGWTServiceAsync;
import org.gcube.portlets.user.td.gwtservice.shared.csv.CSVExportSession;
import org.gcube.portlets.user.td.gwtservice.shared.exception.TDGWTSessionExpiredException;
import org.gcube.portlets.user.td.gwtservice.shared.monitor.OperationMonitor;
import org.gcube.portlets.user.td.gwtservice.shared.monitor.OperationMonitorSession;
import org.gcube.portlets.user.td.monitorwidget.client.MonitorBarUpdater;
import org.gcube.portlets.user.td.monitorwidget.client.MonitorUpdater;
import org.gcube.portlets.user.td.monitorwidget.client.MonitorUpdaterListener;
import org.gcube.portlets.user.td.widgetcommonevent.client.event.SessionExpiredEvent;
import org.gcube.portlets.user.td.widgetcommonevent.client.type.SessionExpiredType;
import org.gcube.portlets.user.td.widgetcommonevent.shared.TRId;
import org.gcube.portlets.user.td.wizardwidget.client.WizardCard;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.FlexTable;
import com.sencha.gxt.core.client.util.Margins;
import com.sencha.gxt.widget.core.client.FramedPanel;
import com.sencha.gxt.widget.core.client.ProgressBar;
import com.sencha.gxt.widget.core.client.box.AlertMessageBox;
import com.sencha.gxt.widget.core.client.container.BoxLayoutContainer.BoxLayoutData;
import com.sencha.gxt.widget.core.client.container.VBoxLayoutContainer;
import com.sencha.gxt.widget.core.client.container.VBoxLayoutContainer.VBoxLayoutAlign;
import com.sencha.gxt.widget.core.client.event.HideEvent;
import com.sencha.gxt.widget.core.client.event.HideEvent.HideHandler;

/**
 * 
 * @author "Giancarlo Panichi" <a
 *         href="mailto:g.panichi@isti.cnr.it">g.panichi@isti.cnr.it</a>
 * 
 */
public class CSVOperationInProgressCard extends WizardCard implements
	MonitorUpdaterListener {

	public static final int STATUS_POLLING_DELAY = 1000;
	protected CSVOperationInProgressCard thisCard;
	protected CSVExportSession exportSession;
	protected MonitorUpdater progressUpdater;
	protected ProgressBar progressBar;

	public CSVOperationInProgressCard(final CSVExportSession exportSession) {
		super("Operation In Progress", "");

		this.exportSession = exportSession;
		thisCard = this;

		VBoxLayoutContainer operationInProgressPanel = new VBoxLayoutContainer();
		operationInProgressPanel.setVBoxLayoutAlign(VBoxLayoutAlign.CENTER);

		final FlexTable description = new FlexTable();
		// FlexCellFormatter cellFormatter = description.getFlexCellFormatter();
		description.setCellSpacing(10);
		description.setCellPadding(4);
		description.setBorderWidth(0);

		// display:block;vertical-align:text-top;
		description.setHTML(0, 0,
				"<span style=\"font-weight:bold;\";>Destination: </span>");
		description.setText(0, 1, exportSession.getDestination().getName());
		description.setHTML(1, 0,
				"<span style=\"font-weight:bold;\";>File Name: </span>");
		description.setText(1, 1, exportSession.getFileName());

		description.setHTML(2, 0,
				"<span style=\"font-weight:bold;\";>File Description: </span>");
		description.setText(2, 1, exportSession.getFileDescription());

		FramedPanel summary = new FramedPanel();
		summary.setHeadingText("Export Summary");
		summary.setWidth(400);
		summary.add(description);
		operationInProgressPanel.add(summary, new BoxLayoutData(new Margins(20,
				5, 10, 5)));

		ProgressBar progressBar = new ProgressBar();
		operationInProgressPanel.add(progressBar, new BoxLayoutData(
				new Margins(10, 5, 10, 5)));
		

		
		setContent(operationInProgressPanel);

	}

	// columnToImportMask

	public void exportCSV() {
		TDGWTServiceAsync.INSTANCE.startCSVExport(exportSession,
				new AsyncCallback<String>() {

					public void onSuccess(String taskId) {
						OperationMonitorSession operationMonitorSession=new OperationMonitorSession(taskId);
						progressUpdater = new MonitorUpdater(operationMonitorSession);
						progressUpdater
								.addListener(new MonitorBarUpdater(progressBar));

						progressUpdater.addListener(thisCard);
						progressUpdater.scheduleRepeating(STATUS_POLLING_DELAY);
					
					}

					public void onFailure(Throwable caught) {
						if (caught instanceof TDGWTSessionExpiredException) {
							getEventBus()
									.fireEvent(
											new SessionExpiredEvent(
													SessionExpiredType.EXPIREDONSERVER));
						} else {
							showErrorAndHide(
									"Error in exportCSV",
									"An error occured in exportCSV: "
											+ caught.getLocalizedMessage(),
									caught.getStackTrace().toString(), caught);
						}
					}
				});
	}

	@Override
	public void setup() {
		getWizardWindow().setEnableBackButton(false);
		setBackButtonVisible(false);
		setNextButtonVisible(false);
		getWizardWindow().setEnableNextButton(false);
		getWizardWindow().setNextButtonToFinish();
		exportCSV();
	}

	
	@Override
	public void monitorInitializing() {

	}
	
	@Override
	public void monitorUpdate(float elaborated) {

	}
	
	@Override
	public void monitorComplete(final TRId trId) {
	
		Command sayComplete = new Command() {
			public void execute() {
				try {
					getWizardWindow().close(false);
					Log.info("fire Complete: "+ trId);
	
					getWizardWindow().fireCompleted(trId);

				} catch (Exception e) {
					Log.error("fire Complete :" + e.getLocalizedMessage());
				}
			}
		};

		getWizardWindow().setNextButtonCommand(sayComplete);

		setNextButtonVisible(true);
		getWizardWindow().setEnableNextButton(true);
	}
	
	@Override
	public void monitorFailed(Throwable caught, String reason,
			String failureDetails) {
		if (caught instanceof TDGWTSessionExpiredException) {
			getEventBus()
					.fireEvent(
							new SessionExpiredEvent(
									SessionExpiredType.EXPIREDONSERVER));
		} else {
			AlertMessageBox d = new AlertMessageBox("Error in Codelist Mapping Import",
					reason);
			d.addHideHandler(new HideHandler() {
				public void onHide(HideEvent event) {
				}
			});
			d.show();
		}
	}

	@Override
	public void monitorStopped(final TRId trId, String reason, String details) {
	
		Command sayComplete = new Command() {
			public void execute() {
				try {
					getWizardWindow().close(false);
					Log.info("fire Complete: tabular resource " + trId.getId());
					Log.info("fire Complete: tableId " + trId.getTableId());

					getWizardWindow().fireCompleted(trId);

				} catch (Exception e) {
					Log.error("fire Complete :" + e.getLocalizedMessage());
				}
			}
		};

		getWizardWindow().setNextButtonCommand(sayComplete);

		setNextButtonVisible(true);
		getWizardWindow().setEnableNextButton(true);

	}

	@Override
	public void monitorGeneratingView() {
		// TODO Auto-generated method stub

	}

	@Override
	public void monitorValidate(float elaborated) {
		// TODO Auto-generated method stub

	}


	@Override
	public void monitorChanged(OperationMonitor operationMonitor) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void monitorAborted() {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void monitorPutInBackground() {
		// TODO Auto-generated method stub
		
	}


}
